"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultCloverDevice = void 0;
var sdk = require("remote-pay-cloud-api");
var RemoteMessageParser_1 = require("../../../json/RemoteMessageParser");
var CloverDevice_1 = require("./CloverDevice");
var CloverConnector_1 = require("../CloverConnector");
var Constants_1 = require("../../../util/Constants");
var Logger_1 = require("../util/Logger");
var Version_1 = require("../../../Version");
var remote_pay_cloud_api_1 = require("remote-pay-cloud-api");
var VoidPaymentResponseMessage = remote_pay_cloud_api_1.remotemessage.VoidPaymentResponseMessage;
/**
 * Default Clover Device
 *
 * This is a default implementation of the clover device.
 */
var DefaultCloverDevice = /** @class */ (function (_super) {
    __extends(DefaultCloverDevice, _super);
    function DefaultCloverDevice(configuration) {
        var _this = _super.call(this, configuration.getMessagePackageName(), configuration.getCloverTransport(), configuration.getApplicationId()) || this;
        _this.logger = Logger_1.Logger.create();
        _this.messageParser = RemoteMessageParser_1.RemoteMessageParser.getDefaultInstance();
        _this._remoteMessageVersion = DefaultCloverDevice.DEFAULT_REMOTE_MESSAGE_VERSION;
        _this.msgIdToTask = {};
        // Timer id for the heartbeat loop.  This state is required so that we can clear the timeout and stop the heartbeat check when we are not connected to the device.
        _this.heartbeatTimer = null;
        // Timer id for the heartbeat response.  This state is required so that we do not have more than one ongoing heartbeat request.  The timeout is cleared when a PONG is received,
        // thus allowing another heartbeat check.
        _this.heartbeatResponseTimer = null;
        // Timer id for the reconnect loop.  This state is required so that we can clear the timeout and stop the reconnect loop when we are connected to the device.
        _this.reconnectTimer = null;
        // Flag that prevents multiple unresolved reconnect attempts.  Reset in onConnectionAttemptComplete.
        _this.reconnecting = false;
        _this.heartbeatIntervalInMillis = null;
        _this.heartbeatDisconnectTimeoutInMillis = null;
        _this.cloverDeviceConfiguration = configuration;
        _this.imageUtil = configuration.getImageUtil();
        _this.maxMessageSizeInChars = Math.max(1000, configuration.getMaxMessageCharacters());
        _this.transport.subscribe(_this);
        _this.transport.setObjectMessageSender(_this);
        _this.reconnectDelayInMillis = _this.cloverDeviceConfiguration["getReconnectDelay"] ? _this.cloverDeviceConfiguration["getReconnectDelay"]() : -1;
        _this.heartbeatIntervalInMillis = _this.cloverDeviceConfiguration["getHeartbeatInterval"] ? _this.cloverDeviceConfiguration["getHeartbeatInterval"]() : -1;
        _this.heartbeatDisconnectTimeoutInMillis = _this.cloverDeviceConfiguration["getHeartbeatDisconnectTimeout"] ? _this.cloverDeviceConfiguration["getHeartbeatDisconnectTimeout"]() : -1;
        _this.forceConnect = _this.cloverDeviceConfiguration["getForceConnect"] ? _this.cloverDeviceConfiguration["getForceConnect"]() : -1;
        return _this;
    }
    /**
     * @param transport
     * @deprecated - see onConnected.
     */
    DefaultCloverDevice.prototype.onDeviceConnected = function (transport) {
        this.onConnected(transport);
    };
    /**
     * We are connected.  What "connected" means depends on the transport mechanism.
     *
     * For network (SNPD) this means that we have connected to the Clover device.
     * For cloud (CPD) this means that we have connected to the cloud proxy.
     */
    DefaultCloverDevice.prototype.onConnected = function (transport) {
        var _this = this;
        // We must initiate the heartbeat to allow non-direct transports' reconnect logic to work upon.
        // initial connection.  If the device is not connected to the proxy and we were initiating the heartbeat
        // in onDeviceReady the heartbeat would never be initiated and the reconnect logic wouldn't work,
        // resulting in a failure to connect and no retries.  If we don't do this with a delay the initial
        // connection attempt may not be complete and we may send two discovery requests.
        setTimeout(function () { return _this.initiateHeartbeat(); }, DefaultCloverDevice.INITIAL_HEARTBEAT_DELAY);
        this.notifyObserversConnected(transport);
    };
    /**
     * The connection attempt is complete.  Set the reconnecting flag to false so that the reconnect loop can try again (if running).
     * @param transport
     */
    DefaultCloverDevice.prototype.onConnectionAttemptComplete = function (transport) {
        this.reconnecting = false;
    };
    /**
     * @param transport
     * @deprecated - see onReady.
     */
    DefaultCloverDevice.prototype.onDeviceReady = function (transport) {
        this.onReady(transport);
    };
    /**
     * We are ready to send messages.  This has different meanings depending on the transport mechanism.
     *
     * For network (SNPD) this means that we have connected to and successfully pinged the Clover device.
     * For cloud (CPD) this means that we have connected to and successfully pinged the cloud proxy.
     *
     * This is generally used to indicate that we are clear to initiate the device via a Discovery Request.
     *
     * Note: this does not mean the device is ready to take a payment through the SDK, which is solely determined
     * by the receipt of a Discovery Response (see DefaultCloverDevice.notifyObserversReady).
     */
    DefaultCloverDevice.prototype.onReady = function (transport) {
        this.doDiscoveryRequest();
    };
    /**
     * Executes a device heartbeat check (via PING) when we are connected to the device. If a PING request is not answered
     * within this.heartbeatResponseTimer disconnect will be called and the SDK will start reconnect attempts.
     */
    DefaultCloverDevice.prototype.initiateHeartbeat = function () {
        var _this = this;
        if (this.heartbeatIntervalInMillis === -1) {
            this.logger.info(new Date().toISOString() + " - Device heartbeat checks are disabled, the heartbeatInterval is set to -1.");
            return;
        }
        if (this.heartbeatTimer) {
            return; // A heartbeatTimer already exists, don't create another.
        }
        var performHeartbeat = function () {
            try {
                if (!_this.heartbeatResponseTimer) {
                    _this.logger.info(new Date().toISOString() + " - Executing device heartbeat check ...");
                    _this.sendPingToDevice();
                    _this.heartbeatResponseTimer = setTimeout(function () {
                        var disconnectMessage = "Disconnecting: We have not received a heartbeat response from the device in " + _this.heartbeatDisconnectTimeoutInMillis + " millis.";
                        _this.logger.warn(new Date().toISOString() + " - " + disconnectMessage);
                        _this.onDisconnected(_this.transport, disconnectMessage);
                    }, _this.heartbeatDisconnectTimeoutInMillis);
                }
                else {
                    _this.logger.info(new Date().toISOString() + " - A heartbeat request is already outstanding, this interval will be skipped.");
                }
                // Schedule future heartbeats.
                _this.heartbeatTimer = setTimeout(performHeartbeat, _this.heartbeatIntervalInMillis);
            }
            catch (e) {
                _this.logger.info(new Date().toISOString() + " - Error caught executing device heartbeat checks.  Message: " + e.message + ".");
            }
        };
        // First time in, perform the heartbeat immediately.
        performHeartbeat();
    };
    DefaultCloverDevice.prototype.stopHeartbeat = function () {
        if (this.heartbeatIntervalInMillis !== -1) {
            this.logger.info(new Date().toISOString() + " - Stopping device heartbeat checks.");
            clearTimeout(this.heartbeatTimer);
            this.heartbeatTimer = null;
            this.clearHeartbeartResponseTimer();
        }
    };
    DefaultCloverDevice.prototype.clearHeartbeartResponseTimer = function () {
        clearTimeout(this.heartbeatResponseTimer);
        this.heartbeatResponseTimer = null;
    };
    /**
     * Executes a device reconnect when we are not connected to the device and reconnect is enabled.
     *
     * The reconnect logic has been moved from the websocket transport layer to the device level to support non-direct connection
     * transports (e.g. cloud).  For non-direct transports the transport layer does not tell the entire truth about the connection
     * status as it only indicates the SDKs connection to the proxy layer.  In order to accurately determine the connection status
     * to the device we must rely on the Discovery Response (notifyObserversReady) and a device PING/PONG (see pingDevice).
     */
    DefaultCloverDevice.prototype.initiateReconnect = function () {
        var _this = this;
        if (this.reconnectDelayInMillis === -1) {
            this.logger.info(new Date().toISOString() + " - Device reconnection is disabled, the reconnectDelay is set to -1.");
            return;
        }
        if (!this.transport || this.transport.isShutdown()) {
            return; // The transport is shutdown, the connector has been disposed.
        }
        if (this.reconnectTimer) {
            return; // A reconnectTimer already exists, don't create another.
        }
        var performReconnect = function () {
            try {
                if (_this.transport && !_this.transport.isShutdown()) {
                    if (!_this.reconnecting) {
                        _this.logger.info(new Date().toISOString() + " - Not connected to your Clover device.  Attempting to reconnect now ...");
                        _this.transport.initialize();
                        _this.reconnecting = true;
                    }
                    else {
                        _this.logger.debug(new Date().toISOString() + " - A reconnection attempt is already outstanding, this attempt will be skipped.");
                    }
                }
            }
            catch (e) {
                _this.logger.error(new Date().toISOString() + " - An exception was caught in the reconnect loop.  Message: " + e.message + ".");
            }
            // Schedule future reconnect attempts.
            _this.reconnectTimer = setTimeout(performReconnect, _this.reconnectDelayInMillis);
        };
        // First time in, perform the reconnect attempt immediately.
        performReconnect();
    };
    DefaultCloverDevice.prototype.stopReconnect = function () {
        if (this.reconnectDelayInMillis !== -1) {
            this.logger.info(new Date().toISOString() + " - Stopping reconnect loop.");
            clearTimeout(this.reconnectTimer);
            this.reconnectTimer = null;
        }
    };
    /**
     * @param transport - the way to notify is defined by the transport
     * @param message - the optional message to send
     * @deprecated - see onDisconnected.
     */
    DefaultCloverDevice.prototype.onDeviceDisconnected = function (transport, message) {
        this.onDisconnected(transport, message);
    };
    /**
     * We are disconnected.  What "disconnected" means depends on the transport mechanism.
     *
     * For network (SNPD) this means that we have disconnected from the Clover device.
     * For cloud (CPD) this means that we have disconnected from the cloud proxy.
     */
    DefaultCloverDevice.prototype.onDisconnected = function (transport, message, reconnect) {
        if (reconnect === void 0) { reconnect = true; }
        // For CPD if we already have a device connected don't attempt reconnect
        if (!message || message.indexOf(Constants_1.Constants.device_already_connected) == -1) {
            this.stopHeartbeat(); // We are offline, kill the heartbeat.
            this.reconnecting = false;
            if (reconnect) {
                this.initiateReconnect();
            }
            this.notifyObserversDisconnected(transport, message);
        }
    };
    DefaultCloverDevice.prototype.onDeviceError = function (deviceError) {
        // A deviceError code of sdk.remotepay.DeviceErrorEventCode.AccessDenied indicates that another
        // POS is already connected to the Clover Device. In this case we want to disable the reconnect loop.
        if (deviceError.getCode() && deviceError.getCode() === sdk.remotepay.DeviceErrorEventCode.AccessDenied) {
            this.stopReconnect();
        }
        this.notifyObserversDeviceError(deviceError);
    };
    DefaultCloverDevice.prototype.getApplicationId = function () {
        return this.applicationId;
    };
    DefaultCloverDevice.prototype.handleRemoteMessagePING = function () {
        this.respondToDevicePing();
    };
    DefaultCloverDevice.prototype.handleRemoteMessagePONG = function () {
        this.clearHeartbeartResponseTimer();
        this.logger.debug("Received pong " + new Date().toISOString());
    };
    Object.defineProperty(DefaultCloverDevice.prototype, "remoteMessageVersion", {
        get: function () {
            return this._remoteMessageVersion;
        },
        /**
         * Remote Message version is used for high-level feature detection e.g. is chunking supported.
         * We set the remote version when incoming messages are handled (handleRemoteMessageCOMMAND).
         * We only want to set _remoteMessageVersion if the inbound message is > than the version already set.
         *
         * @param {number} remoteMessageVersion
         */
        set: function (remoteMessageVersion) {
            if (remoteMessageVersion > this._remoteMessageVersion) {
                this._remoteMessageVersion = remoteMessageVersion;
            }
        },
        enumerable: false,
        configurable: true
    });
    DefaultCloverDevice.prototype.handleRemoteMessageCOMMAND = function (rMessage) {
        this.remoteMessageVersion = typeof rMessage["getVersion"] === "function" ? rMessage.getVersion() : DefaultCloverDevice.DEFAULT_REMOTE_MESSAGE_VERSION;
        var method = sdk.remotemessage.Method[rMessage.getMethod()];
        if (method == null) {
            this.logger.error('Unsupported method type: ' + rMessage.getMethod());
        }
        else {
            var sdkMessage = this.messageParser.parseMessageFromRemoteMessageObj(rMessage);
            if (sdkMessage == null) {
                this.logger.error('Error parsing message: ' + JSON.stringify(rMessage));
            }
            switch (method) {
                case sdk.remotemessage.Method.BREAK:
                    break;
                case sdk.remotemessage.Method.CASHBACK_SELECTED:
                    this.notifyObserversCashbackSelected(sdkMessage);
                    break;
                case sdk.remotemessage.Method.ACK:
                    this.notifyObserverAck(sdkMessage);
                    break;
                case sdk.remotemessage.Method.DISCOVERY_RESPONSE:
                    this.logger.debug('Got a Discovery Response');
                    this.notifyObserversReady(this.transport, sdkMessage);
                    break;
                case sdk.remotemessage.Method.CONFIRM_PAYMENT_MESSAGE:
                    this.notifyObserversConfirmPayment(sdkMessage);
                    break;
                case sdk.remotemessage.Method.FINISH_CANCEL:
                    this.notifyObserversFinishCancel(sdkMessage);
                    break;
                case sdk.remotemessage.Method.FINISH_OK:
                    this.notifyObserversFinishOk(sdkMessage);
                    break;
                case sdk.remotemessage.Method.KEY_PRESS:
                    this.notifyObserversKeyPressed(sdkMessage);
                    break;
                case sdk.remotemessage.Method.ORDER_ACTION_RESPONSE:
                    break;
                case sdk.remotemessage.Method.PARTIAL_AUTH:
                    this.notifyObserversPartialAuth(sdkMessage);
                    break;
                case sdk.remotemessage.Method.VOID_PAYMENT_RESPONSE:
                    this.notifyObserversPaymentVoided(sdkMessage);
                    break;
                case sdk.remotemessage.Method.VOID_PAYMENT_REFUND_RESPONSE:
                    this.notifyObserversPaymentRefundVoided(sdkMessage);
                    break;
                case sdk.remotemessage.Method.PAYMENT_VOIDED:
                    // currently this only gets called during a TX, so falls outside our current process flow
                    //PaymentVoidedMessage vpMessage = (PaymentVoidedMessage) Message.fromJsonString(rMessage.payload);
                    //this.notifyObserversPaymentVoided(vpMessage.payment, vpMessage.voidReason, ResultStatus.SUCCESS, null, null);
                    break;
                case sdk.remotemessage.Method.TIP_ADDED:
                    this.notifyObserversTipAdded(sdkMessage);
                    break;
                case sdk.remotemessage.Method.TX_START_RESPONSE:
                    this.notifyObserverTxStart(sdkMessage);
                    break;
                case sdk.remotemessage.Method.TX_STATE:
                    this.notifyObserversTxState(sdkMessage);
                    break;
                case sdk.remotemessage.Method.UI_STATE:
                    this.notifyObserversUiState(sdkMessage);
                    break;
                case sdk.remotemessage.Method.VERIFY_SIGNATURE:
                    this.notifyObserversVerifySignature(sdkMessage);
                    break;
                case sdk.remotemessage.Method.REFUND_RESPONSE:
                    this.notifyObserversPaymentRefundResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.REFUND_REQUEST:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.TIP_ADJUST_RESPONSE:
                    this.notifyObserversTipAdjusted(sdkMessage);
                    break;
                case sdk.remotemessage.Method.VAULT_CARD_RESPONSE:
                    this.notifyObserverVaultCardResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.CAPTURE_PREAUTH_RESPONSE:
                    this.notifyObserversCapturePreAuth(sdkMessage);
                    break;
                case sdk.remotemessage.Method.INCREMENT_PREAUTH_RESPONSE:
                    this.notifyObserversIncrementPreAuth(sdkMessage);
                    break;
                case sdk.remotemessage.Method.CLOSEOUT_RESPONSE:
                    this.notifyObserversCloseout(sdkMessage);
                    break;
                case sdk.remotemessage.Method.RETRIEVE_PENDING_PAYMENTS_RESPONSE:
                    this.notifyObserversPendingPaymentsResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.CARD_DATA_RESPONSE:
                    this.notifyObserversReadCardData(sdkMessage);
                    break;
                case sdk.remotemessage.Method.ACTIVITY_MESSAGE_FROM_ACTIVITY:
                    this.notifyObserverActivityMessage(sdkMessage);
                    break;
                case sdk.remotemessage.Method.DISCOVERY_REQUEST:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.ORDER_ACTION_ADD_DISCOUNT:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.ORDER_ACTION_ADD_LINE_ITEM:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.ORDER_ACTION_REMOVE_LINE_ITEM:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.ORDER_ACTION_REMOVE_DISCOUNT:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.PRINT_IMAGE:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.PRINT_TEXT:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.PRINT_CREDIT:
                    this.notifyObserversPrintCredit(sdkMessage);
                    break;
                case sdk.remotemessage.Method.PRINT_CREDIT_DECLINE:
                    this.notifyObserversPrintCreditDecline(sdkMessage);
                    break;
                case sdk.remotemessage.Method.PRINT_PAYMENT:
                    this.notifyObserversPrintPayment(sdkMessage);
                    break;
                case sdk.remotemessage.Method.PRINT_PAYMENT_DECLINE:
                    this.notifyObserversPrintPaymentDecline(sdkMessage);
                    break;
                case sdk.remotemessage.Method.PRINT_PAYMENT_MERCHANT_COPY:
                    this.notifyObserversPrintMerchantCopy(sdkMessage);
                    break;
                case sdk.remotemessage.Method.REFUND_PRINT_PAYMENT:
                    this.notifyObserversPrintPaymentRefund(sdkMessage);
                    break;
                case sdk.remotemessage.Method.ACTIVITY_RESPONSE:
                    this.notifyObserversActivityResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.REMOTE_ERROR:
                    this.notifyObserversRemoteError(sdkMessage);
                    break;
                case sdk.remotemessage.Method.RETRIEVE_DEVICE_STATUS_RESPONSE:
                    this.notifyObserversRetrieveDeviceStatusResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.RESET_DEVICE_RESPONSE:
                    this.notifyObserversResetDeviceResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.RETRIEVE_PAYMENT_RESPONSE:
                    this.notifyObserversRetrievePaymentResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.GET_PRINTERS_RESPONSE:
                    this.notifyObserversRetrievePrintersResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.PRINT_JOB_STATUS_RESPONSE:
                    this.notifyObserversPrintJobStatusResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.CUSTOMER_PROVIDED_DATA_MESSAGE:
                    this.notifyObserversCustomerProvidedDataMessage(sdkMessage);
                    break;
                case sdk.remotemessage.Method.INVALID_STATE_TRANSITION:
                    this.notifyObserversInvalidStateTransitionResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.SHOW_RECEIPT_OPTIONS_RESPONSE:
                    this.notifyObserverDisplayReceiptOptionsResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.REQUEST_SIGNATURE_RESPONSE:
                    this.notifyObserverDisplayCollectedSignature(sdkMessage);
                    break;
                case sdk.remotemessage.Method.BALANCE_INQUIRY_RESPONSE:
                    this.notifyObserverBalanceInquiryResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.REQUEST_TIP_RESPONSE: // this is NOT a tip in a payment flow, but for a stanalone tip request view REQUEST_TIP
                    this.notifyObserverTipResponse(sdkMessage);
                    break;
                case sdk.remotemessage.Method.SHOW_ORDER_SCREEN:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.SHOW_THANK_YOU_SCREEN:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.SHOW_WELCOME_SCREEN:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.SIGNATURE_VERIFIED:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.TERMINAL_MESSAGE:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.TX_START:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.VOID_PAYMENT:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.CAPTURE_PREAUTH:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.LAST_MSG_REQUEST:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.LAST_MSG_RESPONSE:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.TIP_ADJUST:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.OPEN_CASH_DRAWER:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.SHOW_PAYMENT_RECEIPT_OPTIONS:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.VAULT_CARD:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.CLOSEOUT_REQUEST:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.ACTIVITY_REQUEST:
                    //Outbound no-op
                    break;
                case sdk.remotemessage.Method.RETRIEVE_PAYMENT_REQUEST:
                    //Outbound no-op
                    break;
                default:
                    this.logger.error('COMMAND not supported with method: ' + rMessage.getMethod());
                    break;
            }
        }
    };
    DefaultCloverDevice.prototype.handleRemoteMessageQUERY = function (rMessage) {
        // no-op
    };
    DefaultCloverDevice.prototype.handleRemoteMessageEVENT = function (rMessage) {
        // no-op
    };
    DefaultCloverDevice.prototype.handleRemoteMessage = function (rMessage) {
        try {
            var msgType = rMessage.getType();
            if (msgType == sdk.remotemessage.RemoteMessageType.PING) {
                this.handleRemoteMessagePING();
            }
            else if (msgType == sdk.remotemessage.RemoteMessageType.PONG) {
                this.handleRemoteMessagePONG();
            }
            else if (msgType == sdk.remotemessage.RemoteMessageType.COMMAND) {
                this.handleRemoteMessageCOMMAND(rMessage);
            }
            else if (msgType == sdk.remotemessage.RemoteMessageType.QUERY) {
                this.handleRemoteMessageQUERY(rMessage);
            }
            else if (msgType == sdk.remotemessage.RemoteMessageType.EVENT) {
                this.handleRemoteMessageEVENT(rMessage);
            }
            else {
                this.logger.error('Unsupported message type: ' + rMessage && rMessage["getType"] ? rMessage.getType() : "Message type unavailable" + " message: " + JSON.stringify(rMessage));
            }
        }
        catch (eM) {
            this.logger.error('Error processing message: ' + rMessage.getPayload(), eM);
        }
    };
    /**
     * Called when a raw message is received from the device
     *
     * @param {string} message - the raw message from the device
     */
    DefaultCloverDevice.prototype.onMessage = function (message) {
        this.logger.debug("DefaultCloverDevice, handling remote message receipt.  Message: " + message + ".");
        try {
            // The cloud proxy sends two versions of the force connect message.  The new SDK can't parse and doesn't need to worry about the old one.
            var isLegacyForceConnect = function (messageIn) {
                return messageIn.indexOf("method") === -1 && messageIn.indexOf("forceConnect") > -1;
            };
            if (isLegacyForceConnect(message)) {
                this.logger.debug('onMessage: Received a legacy force connect message, dropping.');
                return;
            }
            var rMessage = this.messageParser.parseToRemoteMessage(message);
            this.handleRemoteMessage(rMessage);
        }
        catch (e) {
            this.logger.error(e);
        }
    };
    /**
     * Send a PONG response
     */
    DefaultCloverDevice.prototype.respondToDevicePing = function () {
        if (this.transport) {
            var remoteMessage = new sdk.remotemessage.RemoteMessage();
            remoteMessage.setType(sdk.remotemessage.RemoteMessageType.PONG);
            remoteMessage.setPackageName(this.packageName);
            remoteMessage.setRemoteSourceSDK(DefaultCloverDevice.REMOTE_SDK);
            remoteMessage.setRemoteApplicationID(this.applicationId);
            this.sendRemoteMessage(remoteMessage);
        }
        else {
            this.logger.info("Cannot respond to PING, the transport has been shutdown.");
        }
    };
    /**
     * Send a PING message
     */
    DefaultCloverDevice.prototype.sendPingToDevice = function () {
        if (this.transport) {
            var remoteMessage = new sdk.remotemessage.RemoteMessage();
            remoteMessage.setType(sdk.remotemessage.RemoteMessageType.PING);
            remoteMessage.setPackageName(this.packageName);
            remoteMessage.setRemoteSourceSDK(DefaultCloverDevice.REMOTE_SDK);
            remoteMessage.setRemoteApplicationID(this.applicationId);
            this.sendRemoteMessage(remoteMessage);
        }
        else {
            this.logger.info("Cannot send PING, the transport has been shutdown.");
        }
    };
    /**
     * Notify the observers that the device is connected
     *
     * @param transport
     */
    DefaultCloverDevice.prototype.notifyObserversConnected = function (transport) {
        var _this = this;
        this.deviceObservers.forEach(function (obs) {
            obs.onDeviceConnected(_this);
        });
    };
    /**
     * Notify the observers that the device has disconnected
     *
     * @param transport
     * @param message
     */
    DefaultCloverDevice.prototype.notifyObserversDisconnected = function (transport, message) {
        var _this = this;
        this.deviceObservers.forEach(function (obs) {
            obs.onDeviceDisconnected(_this, message);
        });
    };
    /**
     * Notify the observers that the transport failed.
     *
     * @param errorEvent
     */
    DefaultCloverDevice.prototype.notifyObserversDeviceError = function (errorEvent) {
        this.deviceObservers.forEach(function (obs) {
            obs.onDeviceError(errorEvent);
        });
    };
    /**
     * Notify the observers that the device is ready
     *
     * @param transport
     * @param drm
     */
    DefaultCloverDevice.prototype.notifyObserversReady = function (transport, drm) {
        var _this = this;
        if (drm.getReady()) {
            this.stopReconnect();
            this.clearHeartbeartResponseTimer();
            this.initiateHeartbeat();
        }
        this.deviceObservers.forEach(function (obs) {
            obs.onDeviceReady(_this, drm);
        });
    };
    DefaultCloverDevice.prototype.notifyObserverAck = function (ackMessage) {
        var ackTask = this.msgIdToTask[ackMessage.getSourceMessageId()];
        if (ackTask) {
            delete this.msgIdToTask[ackMessage.getSourceMessageId()];
            ackTask.call(null);
        }
        // go ahead and notify listeners of the ACK
        this.deviceObservers.forEach(function (obs) {
            obs.onMessageAck(ackMessage.getSourceMessageId());
        });
    };
    DefaultCloverDevice.prototype.notifyObserverActivityMessage = function (activityMessageFromActivity) {
        this.deviceObservers.forEach(function (obs) {
            obs.onMessageFromActivity(activityMessageFromActivity.getAction(), activityMessageFromActivity.getPayload());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversActivityResponse = function (activityResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            var status = activityResponseMessage.getResultCode() == -1 ?
                sdk.remotemessage.ResultStatus.SUCCESS :
                sdk.remotemessage.ResultStatus.CANCEL;
            obs.onActivityResponse(status, activityResponseMessage.getPayload(), activityResponseMessage.getFailReason(), activityResponseMessage.getAction());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversReadCardData = function (cardDataResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onReadCardResponse(cardDataResponseMessage.getStatus(), cardDataResponseMessage.getReason(), cardDataResponseMessage.getCardData());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversRetrieveDeviceStatusResponse = function (retrieveDeviceStatusResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onDeviceStatusResponse(retrieveDeviceStatusResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversRetrievePaymentResponse = function (retrievePaymentResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onRetrievePaymentResponse(retrievePaymentResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversRetrievePrintersResponse = function (getPrintersResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onRetrievePrintersResponse(sdk.remotepay.ResponseCode.SUCCESS, getPrintersResponseMessage.getPrinters());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPrintJobStatusResponse = function (printJobStatusResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPrintJobStatusResponse(sdk.remotepay.ResponseCode.SUCCESS, printJobStatusResponseMessage.getExternalPrintJobId(), printJobStatusResponseMessage.getStatus());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversCustomerProvidedDataMessage = function (customerProvidedDataMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onCustomerProvidedDataMessage(sdk.remotepay.ResponseCode.SUCCESS, customerProvidedDataMessage.getEventId(), customerProvidedDataMessage.getConfig(), customerProvidedDataMessage.getData());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPrintCredit = function (creditPrintMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPrintCredit(creditPrintMessage.getCredit());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPrintCreditDecline = function (declineCreditPrintMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPrintCreditDecline(declineCreditPrintMessage.getCredit(), declineCreditPrintMessage.getReason());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPrintPayment = function (paymentPrintMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPrintPayment(paymentPrintMessage.getPayment(), paymentPrintMessage.getOrder());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPrintPaymentDecline = function (declinePaymentPrintMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPrintPaymentDecline(declinePaymentPrintMessage.getPayment(), declinePaymentPrintMessage.getReason());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPrintMerchantCopy = function (paymentPrintMerchantCopyMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPrintMerchantReceipt(paymentPrintMerchantCopyMessage.getPayment());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPrintPaymentRefund = function (refundPaymentPrintMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPrintRefundPayment(refundPaymentPrintMessage.getPayment(), refundPaymentPrintMessage.getOrder(), refundPaymentPrintMessage.getRefund());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversResetDeviceResponse = function (resetDeviceResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onResetDeviceResponse(sdk.remotepay.ResponseCode.SUCCESS, resetDeviceResponseMessage.getReason(), resetDeviceResponseMessage.getState());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversRemoteError = function (remoteError) {
        this.deviceObservers.forEach(function (obs) {
            // todo:  Add remote error
            var deviceErrorEvent = new sdk.remotepay.CloverDeviceErrorEvent();
            deviceErrorEvent.setCode(sdk.remotepay.DeviceErrorEventCode.UnknownError);
            deviceErrorEvent.setMessage(JSON.stringify(remoteError));
            deviceErrorEvent.setType(sdk.remotepay.ErrorType.EXCEPTION);
            obs.onDeviceError(deviceErrorEvent);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPaymentRefundResponse = function (refundResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPaymentRefundResponse(refundResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPrintMessage = function (refundPaymentPrintMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPrintRefundPayment(refundPaymentPrintMessage.getPayment(), refundPaymentPrintMessage.getOrder(), refundPaymentPrintMessage.getRefund());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversKeyPressed = function (keyPress) {
        this.deviceObservers.forEach(function (obs) {
            obs.onKeyPressed(keyPress.getKeyPress());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversCashbackSelected = function (cashbackSelectedMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onCashbackSelected(cashbackSelectedMessage.getCashbackAmount());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversTipAdded = function (tipAddedMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onTipAdded(tipAddedMessage.getTipAmount());
        });
    };
    DefaultCloverDevice.prototype.notifyObserverTxStart = function (txStartResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onTxStartResponse(txStartResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversTipAdjusted = function (tipAdjustResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onAuthTipAdjusted(tipAdjustResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPartialAuth = function (partialAuthMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPartialAuth(partialAuthMessage.getPartialAuthAmount());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPaymentVoided = function (voidPaymentResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPaymentVoided(voidPaymentResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPaymentRefundVoided = function (voidPaymentRefundResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPaymentRefundVoidResponse(voidPaymentRefundResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversVerifySignature = function (verifySignatureMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onVerifySignature(verifySignatureMessage.getPayment(), verifySignatureMessage.getSignature());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversConfirmPayment = function (confirmPaymentMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onConfirmPayment(confirmPaymentMessage.getPayment(), confirmPaymentMessage.getChallenges());
        });
    };
    DefaultCloverDevice.prototype.notifyObserverVaultCardResponse = function (vaultCardResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onVaultCardResponse(vaultCardResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversUiState = function (uiStateMsg) {
        this.deviceObservers.forEach(function (obs) {
            obs.onUiState(uiStateMsg.getUiState(), uiStateMsg.getUiText(), uiStateMsg.getUiDirection(), uiStateMsg.getInputOptions());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversCapturePreAuth = function (capturePreAuthResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onCapturePreAuth(capturePreAuthResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversIncrementPreAuth = function (incrementPreAuthResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onIncrementPreAuthResponse(incrementPreAuthResponseMessage);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversCloseout = function (closeoutResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onCloseoutResponse(closeoutResponseMessage.getStatus(), closeoutResponseMessage.getReason(), closeoutResponseMessage.getBatch());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversPendingPaymentsResponse = function (retrievePendingPaymentsResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onPendingPaymentsResponse(retrievePendingPaymentsResponseMessage.getStatus() == sdk.remotemessage.ResultStatus.SUCCESS, retrievePendingPaymentsResponseMessage.getPendingPaymentEntries());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversTxState = function (txStateMsg) {
        this.deviceObservers.forEach(function (obs) {
            obs.onTxState(txStateMsg.getTxState());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversFinishCancel = function (finishCancelMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onFinishCancel(finishCancelMessage.getRequestInfo());
        });
    };
    DefaultCloverDevice.prototype.notifyObserversFinishOk = function (finishOkMessage) {
        this.deviceObservers.forEach(function (obs) {
            if (finishOkMessage.getPayment()) {
                obs.onFinishOk(finishOkMessage.getPayment(), finishOkMessage.getSignature(), finishOkMessage.getRequestInfo());
            }
            else if (finishOkMessage.getCredit()) {
                obs.onFinishOk(finishOkMessage.getCredit());
            }
            else if (finishOkMessage.getRefund()) {
                obs.onFinishOk(finishOkMessage.getRefund());
            }
        });
    };
    DefaultCloverDevice.prototype.notifyObserverDisplayCollectedSignature = function (msg) {
        this.deviceObservers.forEach(function (obs) {
            obs.onSignatureCollected(msg);
        });
    };
    DefaultCloverDevice.prototype.notifyObserverBalanceInquiryResponse = function (msg) {
        this.deviceObservers.forEach(function (obs) {
            obs.onBalanceInquiryResponse(msg);
        });
    };
    DefaultCloverDevice.prototype.notifyObserverTipResponse = function (msg) {
        this.deviceObservers.forEach(function (obs) {
            obs.onRequestTipResponse(msg);
        });
    };
    DefaultCloverDevice.prototype.notifyObserversInvalidStateTransitionResponse = function (invalidStateTransitionMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onInvalidStateTransitionResponse(sdk.remotemessage.ResultStatus.CANCEL, invalidStateTransitionMessage.getReason(), invalidStateTransitionMessage.getRequestedTransition(), invalidStateTransitionMessage.getState(), invalidStateTransitionMessage.getData());
        });
    };
    DefaultCloverDevice.prototype.notifyObserverDisplayReceiptOptionsResponse = function (showReceiptOptionsResponseMessage) {
        this.deviceObservers.forEach(function (obs) {
            obs.onDisplayReceiptOptionsResponse(showReceiptOptionsResponseMessage.getStatus(), showReceiptOptionsResponseMessage.getReason());
        });
    };
    /**
     * Show Payment Receipt Screen
     *
     * @param {string} orderId
     * @param {string} paymentId
     */
    DefaultCloverDevice.prototype.doShowPaymentReceiptScreen = function (orderId, paymentId) {
        var showPaymentReceiptOptionsMessage = new sdk.remotemessage.ShowPaymentReceiptOptionsMessage();
        showPaymentReceiptOptionsMessage.setOrderId(orderId);
        showPaymentReceiptOptionsMessage.setPaymentId(paymentId);
        showPaymentReceiptOptionsMessage.setVersion(2);
        this.sendObjectMessage(showPaymentReceiptOptionsMessage);
    };
    DefaultCloverDevice.prototype.doShowReceiptScreen = function (orderId, paymentId, refundId, creditId, disablePrinting) {
        var showReceiptOptionsMessage = new sdk.remotemessage.ShowReceiptOptionsMessage();
        showReceiptOptionsMessage.setOrderId(orderId);
        showReceiptOptionsMessage.setPaymentId(paymentId);
        showReceiptOptionsMessage.setRefundId(refundId);
        showReceiptOptionsMessage.setCreditId(creditId);
        showReceiptOptionsMessage.setDisableCloverPrinting(disablePrinting);
        this.sendObjectMessage(showReceiptOptionsMessage);
    };
    /**
     * Key Press
     *
     * @param {sdk.remotemessage.KeyPress} keyPress
     */
    DefaultCloverDevice.prototype.doKeyPress = function (keyPress) {
        var message = new sdk.remotemessage.KeyPressMessage();
        message.setKeyPress(keyPress);
        this.sendObjectMessage(message);
    };
    /**
     * Show Thank You Screen
     */
    DefaultCloverDevice.prototype.doShowThankYouScreen = function () {
        var message = new sdk.remotemessage.ThankYouMessage();
        this.sendObjectMessage(message);
    };
    /**
     * Show Welcome Screen
     */
    DefaultCloverDevice.prototype.doShowWelcomeScreen = function () {
        var message = new sdk.remotemessage.WelcomeMessage();
        this.sendObjectMessage(message);
    };
    /**
     * Signature Verified
     *
     * @param {sdk.payments.Payment} payment
     * @param {boolean} verified
     */
    DefaultCloverDevice.prototype.doSignatureVerified = function (payment, verified) {
        var message = new sdk.remotemessage.SignatureVerifiedMessage();
        message.setPayment(payment);
        message.setVerified(verified);
        this.sendObjectMessage(message);
    };
    /**
     * Retrieve Pending Payments
     */
    DefaultCloverDevice.prototype.doRetrievePendingPayments = function () {
        var message = new sdk.remotemessage.RetrievePendingPaymentsMessage();
        this.sendObjectMessage(message);
    };
    /**
     * Terminal Message
     *
     * @param {string} text
     */
    DefaultCloverDevice.prototype.doTerminalMessage = function (text) {
        var message = new sdk.remotemessage.TerminalMessage();
        message.setText(text);
        this.sendObjectMessage(message);
    };
    /**
     * Sends request to the clover device to send the log to the clover server
     *
     * @param message The message to display
     */
    DefaultCloverDevice.prototype.doSendDebugLog = function (message) {
        var deviceLogMessage = new sdk.remotemessage.CloverDeviceLogMessage();
        deviceLogMessage.setMessage(message);
        this.sendObjectMessage(deviceLogMessage);
    };
    /**
     * Open Cash Drawer
     *
     * @param {string} reason
     * @param {string} deviceId (optional)
     */
    DefaultCloverDevice.prototype.doOpenCashDrawer = function (reason, deviceId) {
        var message = new sdk.remotemessage.OpenCashDrawerMessage();
        message.setReason(reason);
        if (deviceId) {
            var ptr = new sdk.printer.Printer();
            ptr.setId(deviceId);
            message.setPrinter(ptr);
        }
        this.sendObjectMessage(message);
    };
    /**
     * Closeout
     *
     * @param {boolean} allowOpenTabs
     * @param {string} batchId
     */
    DefaultCloverDevice.prototype.doCloseout = function (allowOpenTabs, batchId) {
        var message = new sdk.remotemessage.CloseoutRequestMessage();
        message.setAllowOpenTabs(allowOpenTabs);
        message.setBatchId(batchId);
        this.sendObjectMessage(message);
    };
    /**
     * Transaction Start
     *
     * @param {sdk.remotemessage.PayIntent} payIntent
     * @param {sdk.remotemessage.Order} order
     * @param {string} requestInfo - request type.
     */
    DefaultCloverDevice.prototype.doTxStart = function (payIntent, order, requestInfo) {
        var message = new sdk.remotemessage.TxStartRequestMessage();
        message.setPayIntent(payIntent);
        message.setOrder(order);
        message.setRequestInfo(requestInfo);
        message.setVersion(2);
        this.sendObjectMessage(message);
    };
    /**
     * Tip Adjust Auth
     *
     * @param {string} orderId
     * @param {string} paymentId
     * @param {number} amount
     */
    DefaultCloverDevice.prototype.doTipAdjustAuth = function (orderId, paymentId, amount) {
        var message = new sdk.remotemessage.TipAdjustMessage();
        message.setOrderId(orderId);
        message.setPaymentId(paymentId);
        message.setTipAmount(amount);
        this.sendObjectMessage(message);
    };
    /**
     * Read Cart Data
     *
     * @param {PayIntent} payIntent
     */
    DefaultCloverDevice.prototype.doReadCardData = function (payIntent) {
        var message = new sdk.remotemessage.CardDataRequestMessage();
        message.setPayIntent(payIntent);
        this.sendObjectMessage(message);
    };
    /**
     * Send a message to a running custom activity
     *
     * @param {string} actionId - the id used when the custom action was started
     * @param {string} payload - the message content, unrestricted format
     */
    DefaultCloverDevice.prototype.doSendMessageToActivity = function (actionId, payload) {
        var message = new sdk.remotemessage.ActivityMessageToActivity();
        message.setAction(actionId);
        message.setPayload(payload);
        this.sendObjectMessage(message);
    };
    /**
     * Print Text
     *
     * @param {Array<string>} textLines
     * @param printRequestId - an optional id that will be used for the printjob.  This id will be used in notification calls about the status of the job.
     * @param printDeviceId - the printer id to use when printing.  If left unset the default is used
     */
    DefaultCloverDevice.prototype.doPrintText = function (textLines, printRequestId, printDeviceId) {
        var message = new sdk.remotemessage.TextPrintMessage();
        message.setTextLines(textLines);
        if (printRequestId) {
            message.setExternalPrintJobId(printRequestId);
        }
        if (printDeviceId) {
            var ptr = new sdk.printer.Printer();
            ptr.setId(printDeviceId);
            message.setPrinter(ptr);
        }
        this.sendObjectMessage(message);
    };
    /**
     * Print Image (Bitmap)
     *
     * @param bitmap
     * @param printRequestId
     * @param printDeviceId
     */
    DefaultCloverDevice.prototype.doPrintImageObject = function (bitmap, printRequestId, printDeviceId) {
        var _this = this;
        var message = new sdk.remotemessage.ImagePrintMessage();
        // bitmap - HTMLImageElement
        this.imageUtil.getBase64Image(bitmap, function (imageString) {
            message.setPng(imageString);
            if (printRequestId) {
                message.setExternalPrintJobId(printRequestId);
            }
            if (printDeviceId) {
                var ptr = new sdk.printer.Printer();
                ptr.setId(printDeviceId);
                message.setPrinter(ptr);
            }
            if (_this.isFragmentationSupported()) {
                // We need to be putting this in the attachment instead of the payload (for the remoteMessage)
                var base64Png = message.getPng();
                message.setPng(null);
                _this.sendObjectMessage(message, base64Png, DefaultCloverDevice.BASE64);
            }
            else {
                _this.sendObjectMessage(message);
            }
        });
    };
    /**
     * Printing images from a url from the device is problematic.
     * See - https://jira.dev.clover.com/browse/SEMI-1352
     * and - https://jira.dev.clover.com/browse/SEMI-1377
     *
     * Instead of relying on the device, we can retrieve the image from the URL
     * and call doPrintImageObject instead of doPrintImageUrl. The doPrintImageObject
     * method is more robust (can handle large images via chunking, etc.).
     *
     * @param {string} url
     * @param {string} printRequestId
     * @param {string} printDeviceId
     */
    DefaultCloverDevice.prototype.doPrintImageUrl = function (url, printRequestId, printDeviceId) {
        var _this = this;
        this.imageUtil.loadImageFromURL(url, function (image) {
            _this.doPrintImageObject(image, printRequestId, printDeviceId);
        }, function (errorMessage) {
            var deviceErrorEvent = new sdk.remotepay.CloverDeviceErrorEvent();
            deviceErrorEvent.setCode(sdk.remotepay.DeviceErrorEventCode.UnknownError);
            deviceErrorEvent.setMessage(errorMessage);
            deviceErrorEvent.setType(sdk.remotepay.ErrorType.EXCEPTION);
            _this.notifyObserversDeviceError(deviceErrorEvent);
        });
    };
    DefaultCloverDevice.prototype.doStartActivity = function (action, payload, nonBlocking) {
        var request = new sdk.remotemessage.ActivityRequest();
        request.setAction(action);
        request.setPayload(payload);
        request.setNonBlocking(nonBlocking);
        request.setForceLaunch(false);
        this.sendObjectMessage(request);
    };
    /**
     * Void Payment
     *
     * @param {sdk.payments.Payment} payment
     * @param {sdk.order.VoidReason} voidReason
     * @param {boolean} disablePrinting
     * @param {boolean} disableReceiptSelection
     */
    DefaultCloverDevice.prototype.doVoidPayment = function (payment, voidReason, disablePrinting, disableReceiptSelection) {
        var _this = this;
        var message = new sdk.remotemessage.VoidPaymentMessage();
        message.setPayment(payment);
        message.setVoidReason(voidReason);
        message.setDisableCloverPrinting(disablePrinting);
        message.setDisableReceiptSelection(disableReceiptSelection);
        if (this.getSupportsVoidPaymentResponse()) {
            message.setVersion(3);
        }
        var remoteMessage = this.buildRemoteMessageToSend(message);
        var msgId = remoteMessage.getId();
        // remote-pay will send the void payment response.
        if (this.getSupportsVoidPaymentResponse()) {
            this.sendRemoteMessage(remoteMessage);
        }
        else {
            var vprm_1 = new VoidPaymentResponseMessage();
            vprm_1.setPayment(payment);
            vprm_1.setVoidReason(voidReason);
            // remote-pay will not-send the void payment response, we will send it here.  Because we don't know the real status
            // of voids the best we can do is to set the status to success.
            vprm_1.setStatus(sdk.remotemessage.ResultStatus.SUCCESS);
            if (!this.getSupportsAck()) {
                this.sendRemoteMessage(remoteMessage);
                this.notifyObserversPaymentVoided(vprm_1);
            }
            else {
                // we will send back response after we get an ack
                this.addTaskForAck(msgId, function () {
                    _this.notifyObserversPaymentVoided(vprm_1);
                });
                this.sendRemoteMessage(remoteMessage);
            }
        }
    };
    DefaultCloverDevice.prototype.addTaskForAck = function (msgId, task) {
        this.msgIdToTask[msgId] = task;
    };
    /**
     * Payment Refund
     *
     * @param {string} orderId
     * @param {string} paymentId
     * @param {number} amount
     * @param {boolean} fullRefund
     * @param {boolean} disablePrinting
     * @param {boolean} disableReceiptSelection
     */
    DefaultCloverDevice.prototype.doPaymentRefund = function (orderId, paymentId, amount, fullRefund, disablePrinting, disableReceiptSelection) {
        var message = new sdk.remotemessage.RefundRequestMessage();
        message.setOrderId(orderId);
        message.setPaymentId(paymentId);
        message.setAmount(amount);
        message.setFullRefund(fullRefund);
        message.setDisableCloverPrinting(disablePrinting);
        message.setDisableReceiptSelection(disableReceiptSelection);
        message.setVersion(2);
        this.sendObjectMessage(message);
    };
    /**
     * Void Payment Refund
     *
     * @param {string} orderId
     * @param {string} refundId
     * @param {boolean} disablePrinting
     * @param {boolean} disableReceiptSelection
     */
    DefaultCloverDevice.prototype.doVoidPaymentRefund = function (orderId, refundId, disablePrinting, disableReceiptSelection) {
        var message = new sdk.remotemessage.VoidPaymentRefundMessage();
        message.setOrderId(orderId);
        message.setRefundId(refundId);
        message.setDisableCloverPrinting(disablePrinting);
        message.setDisableReceiptSelection(disableReceiptSelection);
        message.setVersion(2);
        this.sendObjectMessage(message);
    };
    /**
     * Payment Refund
     *
     * @param {RefundPaymentRequest} request
     */
    DefaultCloverDevice.prototype.doPaymentRefundByRequest = function (request) {
        var message = new sdk.remotemessage.RefundRequestMessage();
        message.setOrderId(request.getOrderId());
        message.setPaymentId(request.getPaymentId());
        message.setAmount(request.getAmount());
        message.setFullRefund(request.getFullRefund());
        message.setDisableCloverPrinting(request.getDisablePrinting());
        message.setDisableReceiptSelection(request.getDisableReceiptSelection());
        message.setVersion(2);
        this.sendObjectMessage(message);
    };
    /**
     * Vault Card
     *
     * @param {number} cardEntryMethods
     */
    DefaultCloverDevice.prototype.doVaultCard = function (cardEntryMethods) {
        var message = new sdk.remotemessage.VaultCardMessage();
        message.setCardEntryMethods(cardEntryMethods);
        this.sendObjectMessage(message);
    };
    /**
     * Capture Auth
     *
     * @param {string} paymentId
     * @param {number} amount
     * @param {number} tipAmount
     */
    DefaultCloverDevice.prototype.doCaptureAuth = function (paymentId, amount, tipAmount) {
        var message = new sdk.remotemessage.CapturePreAuthMessage();
        message.setVersion(1);
        message.setPaymentId(paymentId);
        message.setAmount(amount);
        message.setTipAmount(tipAmount);
        this.sendObjectMessage(message);
    };
    /**
     * Capture Auth
     *
     * @param {string} paymentId
     * @param {number} amount
     * @param {number} amount
     */
    DefaultCloverDevice.prototype.doIncrementPreAuth = function (paymentId, amount) {
        var message = new sdk.remotemessage.IncrementPreAuthMessage();
        message.setVersion(1);
        message.setPaymentId(paymentId);
        message.setAmount(amount);
        this.sendObjectMessage(message);
    };
    /**
     * Accept Payment
     *
     * @param {Payment} payment
     */
    DefaultCloverDevice.prototype.doAcceptPayment = function (payment) {
        var message = new sdk.remotemessage.PaymentConfirmedMessage();
        message.setPayment(payment);
        this.sendObjectMessage(message);
    };
    /**
     * Reject Payment
     *
     * @param {Payment} payment
     * @param {Challenge} challenge
     */
    DefaultCloverDevice.prototype.doRejectPayment = function (payment, challenge) {
        var message = new sdk.remotemessage.PaymentRejectedMessage();
        message.setPayment(payment);
        message.setReason(challenge.getReason());
        this.sendObjectMessage(message);
    };
    /**
     * Discovery request
     */
    DefaultCloverDevice.prototype.doDiscoveryRequest = function () {
        var drm = new sdk.remotemessage.DiscoveryRequestMessage();
        drm.setSupportsOrderModification(false);
        this.sendObjectMessage(drm);
    };
    /**
     * Order Update
     *
     * @param {DisplayOrder} order
     * @param {any} orderOperation
     */
    DefaultCloverDevice.prototype.doOrderUpdate = function (order, orderOperation) {
        var message = new sdk.remotemessage.OrderUpdateMessage();
        message.setOrder(order);
        if (orderOperation) {
            if (orderOperation instanceof sdk.order.operation.DiscountsAddedOperation) {
                message.setDiscountsAddedOperation(orderOperation);
            }
            else if (orderOperation instanceof sdk.order.operation.DiscountsDeletedOperation) {
                message.setDiscountsDeletedOperation(orderOperation);
            }
            else if (orderOperation instanceof sdk.order.operation.LineItemsAddedOperation) {
                message.setLineItemsAddedOperation(orderOperation);
            }
            else if (orderOperation instanceof sdk.order.operation.LineItemsDeletedOperation) {
                message.setLineItemsDeletedOperation(orderOperation);
            }
            else if (orderOperation instanceof sdk.order.operation.OrderDeletedOperation) {
                message.setOrderDeletedOperation(orderOperation);
            }
        }
        this.sendObjectMessage(message);
    };
    /**
     * Reset Device
     */
    DefaultCloverDevice.prototype.doResetDevice = function () {
        var message = new sdk.remotemessage.BreakMessage();
        this.sendObjectMessage(message);
    };
    DefaultCloverDevice.prototype.doRetrieveDeviceStatus = function (request) {
        var message = new sdk.remotemessage.RetrieveDeviceStatusRequestMessage();
        message.setSendLastMessage(request.getSendLastMessage());
        this.sendObjectMessage(message);
    };
    DefaultCloverDevice.prototype.doRetrievePayment = function (externalPaymentId) {
        var message = new sdk.remotemessage.RetrievePaymentRequestMessage();
        message.setExternalPaymentId(externalPaymentId);
        this.sendObjectMessage(message);
    };
    DefaultCloverDevice.prototype.doRetrievePrinters = function (category) {
        var message = new sdk.remotemessage.GetPrintersRequestMessage();
        if (category) {
            message.setCategory(category);
        }
        this.sendObjectMessage(message);
    };
    DefaultCloverDevice.prototype.doRetrievePrintJobStatus = function (printRequestId) {
        var message = new sdk.remotemessage.PrintJobStatusRequestMessage();
        message.setExternalPrintJobId(printRequestId);
        this.sendObjectMessage(message);
    };
    /**
     * Loyalty
     */
    DefaultCloverDevice.prototype.doRegisterForCustomerProvidedData = function (configurations) {
        var message = new sdk.remotemessage.RegisterForCustomerProvidedDataMessage();
        message.setConfigurations(configurations);
        this.sendObjectMessage(message);
    };
    DefaultCloverDevice.prototype.doSetCustomerInfo = function (customerInfo) {
        var message = new sdk.remotemessage.CustomerInfoMessage();
        message.setCustomer(customerInfo);
        this.sendObjectMessage(message);
    };
    DefaultCloverDevice.prototype.doCheckBalance = function (cardEntryMethods) {
        var balanceInquiryRequestMessage = new sdk.remotemessage.BalanceInquiryRequestMessage();
        balanceInquiryRequestMessage.setCardEntryMethods(cardEntryMethods);
        this.sendObjectMessage(balanceInquiryRequestMessage);
    };
    DefaultCloverDevice.prototype.doCollectSignature = function (acknowledge) {
        var signatureRequestMessage = new sdk.remotemessage.SignatureRequestMessage();
        signatureRequestMessage.setAcknowledgementMessage(acknowledge);
        this.sendObjectMessage(signatureRequestMessage);
    };
    DefaultCloverDevice.prototype.doRequestTip = function (tippableAmount, suggestions) {
        var requestTipRequestMessage = new sdk.remotemessage.RequestTipRequestMessage();
        requestTipRequestMessage.setTipSuggestions(suggestions);
        requestTipRequestMessage.setTippableAmount(tippableAmount);
        this.sendObjectMessage(requestTipRequestMessage);
    };
    /**
     * Dispose
     */
    DefaultCloverDevice.prototype.dispose = function () {
        this.deviceObservers.splice(0, this.deviceObservers.length);
        if (this.transport) {
            this.transport.dispose();
            this.transport = null;
        }
        this.stopReconnect(); // must be done after we dispose so the transport is shutdown.
        this.stopHeartbeat();
    };
    DefaultCloverDevice.prototype.sendObjectMessage = function (remoteMessage, attachment, attachmentEncoding) {
        return this.buildRemoteMessages(remoteMessage, attachment, attachmentEncoding); // this now sends the messages and returns the ID
    };
    DefaultCloverDevice.prototype.buildBaseRemoteMessage = function (remoteMessage) {
        // Make sure the message is not null
        if (remoteMessage == null) {
            this.logger.debug('Message is null');
            return null;
        }
        // Check the message method
        if (remoteMessage.getMethod() == null) {
            this.logger.error('Invalid Message', new Error('Invalid Message: ' + remoteMessage.toString()));
            return null;
        }
        // Check the application id
        if (this.applicationId == null) {
            this.logger.error('Invalid ApplicationID: ' + this.applicationId);
            throw new Error('Invalid applicationId');
        }
        var messageId = (++DefaultCloverDevice.id) + '';
        remoteMessage.setMessageId(messageId);
        var remoteMessageToReturn = new sdk.remotemessage.RemoteMessage();
        remoteMessageToReturn.setId(messageId);
        remoteMessageToReturn.setType(sdk.remotemessage.RemoteMessageType.COMMAND);
        remoteMessageToReturn.setPackageName(this.packageName);
        remoteMessageToReturn.setMethod(remoteMessage.getMethod());
        remoteMessageToReturn.setVersion(this.remoteMessageVersion);
        remoteMessageToReturn.setRemoteSourceSDK(DefaultCloverDevice.REMOTE_SDK);
        remoteMessageToReturn.setRemoteApplicationID(this.applicationId);
        return remoteMessageToReturn;
    };
    /**
     * Special serialization handling
     * The top level elements should not have the "elements" wrapper on collections (arrays).
     * sdk.remotemessage.Message instances are the only ones this needs to happen for.  This
     * is the result of the manner in which the serialization/deserialization happens in the
     * Android code.  The top level objects are not (de)serialized by a
     * com.clover.sdk.GenericClient#extractListOther
     * (in the Clover common repo).  The GenericClient is the tool that adds the elements
     * wrapper.  The top level objects are (de)serialized by themselves
     * com.clover.remote.message.Message#fromJsonString
     */
    DefaultCloverDevice.prototype.addSuppressElementsWrapper = function (message) {
        for (var fieldKey in message) {
            // Make sure the message has property and is defined so we can set the `suppressElementsWrapper` flag
            if (message.hasOwnProperty(fieldKey) && message[fieldKey]) {
                var metaInfo = message ? message.getMetaInfo(fieldKey) : null;
                if (metaInfo && (metaInfo.type == Array)) {
                    message[fieldKey].suppressElementsWrapper = true;
                }
            }
        }
        return message;
    };
    DefaultCloverDevice.prototype.buildRemoteMessageToSend = function (message) {
        var remoteMessage = this.buildBaseRemoteMessage(message);
        message = this.addSuppressElementsWrapper(message);
        remoteMessage.setPayload(JSON.stringify(message, DefaultCloverDevice.stringifyClover));
        return remoteMessage;
    };
    DefaultCloverDevice.prototype.buildRemoteMessages = function (message, attachment, attachmentEncoding) {
        var remoteMessage = this.buildBaseRemoteMessage(message);
        message = this.addSuppressElementsWrapper(message);
        if (attachmentEncoding) {
            remoteMessage.setAttachmentEncoding(attachmentEncoding);
        }
        var messagePayload = JSON.stringify(message, DefaultCloverDevice.stringifyClover);
        if (this.isFragmentationSupported()) {
            var payloadTooLarge = (messagePayload ? messagePayload.length : 0) > this.maxMessageSizeInChars;
            if (payloadTooLarge || attachment) { // need to fragment
                if (attachment && attachment.length > CloverConnector_1.CloverConnector.MAX_PAYLOAD_SIZE) {
                    this.logger.error('Error sending message - payload size is greater than the maximum allowed.');
                    return null;
                }
                var fragmentIndex = 0;
                // fragmenting loop for payload
                while (messagePayload.length > 0) {
                    remoteMessage.setLastFragment(false);
                    if (messagePayload.length <= this.maxMessageSizeInChars) {
                        remoteMessage.setPayload(messagePayload);
                        messagePayload = "";
                        // If the attachment is null at this point, then this is the last fragment
                        remoteMessage.setLastFragment(attachment == null);
                    }
                    else {
                        remoteMessage.setPayload(messagePayload.substr(0, this.maxMessageSizeInChars));
                        messagePayload = messagePayload.substr(this.maxMessageSizeInChars);
                    }
                    remoteMessage.setFragmentIndex(fragmentIndex++);
                    this.sendRemoteMessage(remoteMessage);
                } //end fragment payload loop
                remoteMessage.setPayload(null);
                if (attachment) {
                    //fragmenting loop for attachment
                    if (attachmentEncoding == DefaultCloverDevice.BASE64) {
                        remoteMessage.setAttachmentEncoding(DefaultCloverDevice.BASE64_ATTACHMENT);
                        while (attachment.length > 0) {
                            remoteMessage.setLastFragment(false);
                            if (attachment.length <= this.maxMessageSizeInChars) {
                                remoteMessage.setAttachment(attachment);
                                attachment = "";
                                remoteMessage.setLastFragment(true);
                            }
                            else {
                                remoteMessage.setAttachment(attachment.substr(0, this.maxMessageSizeInChars));
                                attachment = attachment.substr(this.maxMessageSizeInChars);
                            }
                            remoteMessage.setFragmentIndex(fragmentIndex++);
                            this.sendRemoteMessage(remoteMessage);
                        } //end fragment attachment loop
                    }
                    else {
                        // We got an attachment, but no encoding, complain.
                        this.logger.error('Attachment on message, but no encoding specified.  No idea how to send it.');
                        // TODO:  Probably a good idea to throw here, but then we need to handle that in the top level.  Leave for later.
                    }
                }
            }
            else { // no need to fragment
                if (messagePayload.length > this.maxMessageSizeInChars) {
                    this.logger.warn("The message payload is larger than the maxMessageSizeInChars but fragmentation is not supported by the apps installed on the device.  This may result in a payload that is too large to handle and a silent failure.");
                }
                remoteMessage.setPayload(messagePayload);
                if (attachment) {
                    remoteMessage.setAttachment(attachment);
                }
                this.sendRemoteMessage(remoteMessage);
            }
        }
        else {
            // fragmenting is not possible, just send as is
            remoteMessage.setPayload(messagePayload);
            this.sendRemoteMessage(remoteMessage);
        }
        return remoteMessage.getId();
    };
    DefaultCloverDevice.stringifyClover = function (key, value) {
        // If the element is an array, and it does NOT have the suppressElementsWrapper property,
        // and the key is NOT "elements", then add the elements wrapper object
        if (Array.isArray(value) &&
            !value.hasOwnProperty("suppressElementsWrapper") &&
            (key != "elements")) {
            //converts array into the format that clover devices expect
            //from) foo : []
            //to) foo : {elements : []}
            return { elements: value };
        }
        return value;
    };
    DefaultCloverDevice.prototype.sendRemoteMessage = function (remoteMessage) {
        var message = JSON.stringify(remoteMessage);
        if (this.transport) {
            this.logger.debug("Sending: " + message);
            this.transport.sendMessage(message);
        }
        else {
            this.logger.error("Cannot send message, transport is null: " + message);
        }
    };
    /**
     * If the remote message version is 2, fragmentation is supported.
     *
     * @returns {boolean}
     */
    DefaultCloverDevice.prototype.isFragmentationSupported = function () {
        return this.remoteMessageVersion > 1;
    };
    // Remote message version and message version are not the same.  Remote message version is used for high-level
    // feature detection - e.g. is message fragmentation supported or not?
    DefaultCloverDevice.DEFAULT_REMOTE_MESSAGE_VERSION = 1;
    DefaultCloverDevice.REMOTE_SDK = Version_1.Version.CLOVER_CLOUD_SDK + ":" + Version_1.Version.CLOVER_CLOUD_SDK_VERSION;
    DefaultCloverDevice.BASE64 = "BASE64";
    DefaultCloverDevice.BASE64_ATTACHMENT = DefaultCloverDevice.BASE64 + ".ATTACHMENT";
    DefaultCloverDevice.id = 0;
    DefaultCloverDevice.INITIAL_HEARTBEAT_DELAY = 15000; // millis
    return DefaultCloverDevice;
}(CloverDevice_1.CloverDevice));
exports.DefaultCloverDevice = DefaultCloverDevice;

//# sourceMappingURL=../../../../../maps/com/clover/remote/client/device/DefaultCloverDevice.js.map
