"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketCloudCloverDeviceConfigurationBuilder = exports.WebSocketCloudCloverDeviceConfiguration = void 0;
var WebSocketCloudCloverTransport_1 = require("../transport/websocket/WebSocketCloudCloverTransport");
var WebsocketCloudCloverDevice_1 = require("./WebsocketCloudCloverDevice");
var WebSocketCloverDeviceConfiguration_1 = require("./WebSocketCloverDeviceConfiguration");
var HttpSupport_1 = require("../../../util/HttpSupport");
var ImageUtil_1 = require("../../../util/ImageUtil");
var BrowserWebSocketImpl_1 = require("../../../websocket/BrowserWebSocketImpl");
/**
 * Configuration used to create a connection to a device via the Clover cloud.
 */
var WebSocketCloudCloverDeviceConfiguration = /** @class */ (function (_super) {
    __extends(WebSocketCloudCloverDeviceConfiguration, _super);
    /**
     *
     * @param {string} applicationId - the applicationId that uniquely identifies the POS.
     *    e.g. com.company.MyPOS:2.3.1 for the first connection
     * @param {Object} webSocketFactoryFunction - the function that will return an instance of the
     *  CloverWebSocketInterface that will be used when connecting.  For Browser implementations, this can be
     *  BrowserWebSocketImpl.createInstance.  For NodeJS implementations, this will be defined differently.
     * @param {IImageUtil} imageUtil - utility to translate images into base64 strings.
     * @param {string} cloverServer the base url for the clover server used in the cloud connection.
     *    EX:  https://www.clover.com, http://localhost:9000
     * @param {string} accessToken - the OAuth access token that will be used when contacting the clover server
     * @param {HttpSupport} httpSupport - the helper object used when making http requests.
     * @param {string} merchantId - the merchant the device belongs to.
     * @param {string} deviceId - the id (not uuid) of the device to connect to
     * @param {string} friendlyId - an identifier for the specific terminal connected to this device.  This id is used
     *  in debugging and may be sent to other clients if they attempt to connect to the same device.  It will also be
     *  sent to other clients that are currently connected if this device does a forceConnect.
     * @param {boolean} forceConnect - if true, overtake any existing connection.
     * @param {number} heartbeatInterval - Frequency at which we will ping the device - millis. Defaults to -1, which means the heartbeat check is disabled.
     * @param {number} reconnectDelay - upon disconnect, duration to wait until a reconnect is attempted - millis. Defaults to 3000 (millis), minimum value is 3000 millis. To disable the reconnect set to -1.
     * @param {number} heartbeatDisconnectTimeout - If a response to a heartbeat ping is not received within this time we will call disconnect.
     */
    function WebSocketCloudCloverDeviceConfiguration(applicationId, webSocketFactoryFunction, imageUtil, cloverServer, accessToken, httpSupport, merchantId, deviceId, friendlyId, forceConnect, heartbeatInterval, reconnectDelay, heartbeatDisconnectTimeout) {
        if (forceConnect === void 0) { forceConnect = false; }
        var _this = _super.call(this, applicationId, webSocketFactoryFunction, imageUtil, heartbeatInterval, reconnectDelay, heartbeatDisconnectTimeout) || this;
        _this.cloverServer = cloverServer;
        _this.accessToken = accessToken;
        _this.httpSupport = httpSupport;
        _this.merchantId = merchantId;
        _this.deviceId = deviceId;
        _this.friendlyId = friendlyId || "unknown";
        _this.forceConnect = forceConnect;
        return _this;
    }
    WebSocketCloudCloverDeviceConfiguration.prototype.getMessagePackageName = function () {
        return 'com.clover.remote.protocol.websocket';
    };
    WebSocketCloudCloverDeviceConfiguration.prototype.getName = function () {
        return 'Clover Cloud WebSocket Connector';
    };
    WebSocketCloudCloverDeviceConfiguration.prototype.getCloverDeviceType = function () {
        return WebsocketCloudCloverDevice_1.WebsocketCloudCloverDevice;
    };
    WebSocketCloudCloverDeviceConfiguration.prototype.getCloverTransport = function () {
        // this is where we determine more about the transport...
        // The connection is paired:
        var transport = new WebSocketCloudCloverTransport_1.WebSocketCloudCloverTransport(this.getReconnectDelay(), this.webSocketImplClass, this.cloverServer, this.merchantId, this.accessToken, this.deviceId, this.friendlyId, this.forceConnect, this.httpSupport);
        return transport;
    };
    return WebSocketCloudCloverDeviceConfiguration;
}(WebSocketCloverDeviceConfiguration_1.WebSocketCloverDeviceConfiguration));
exports.WebSocketCloudCloverDeviceConfiguration = WebSocketCloudCloverDeviceConfiguration;
var WebSocketCloudCloverDeviceConfigurationBuilder = /** @class */ (function () {
    /**
     *
     * @param {string} applicationId - the applicationId that uniquely identifies the POS.
     *    e.g. com.company.MyPOS:2.3.1 for the first connection
     * @param {string} deviceId - the id (not uuid) of the device to connect to
     * @param {string} merchantId - the merchant the device belongs to.
     * @param {string} accessToken - the OAuth access token that will be used when contacting the clover server
     */
    function WebSocketCloudCloverDeviceConfigurationBuilder(applicationId, deviceId, merchantId, accessToken) {
        this.forceConnect = false;
        this.applicationId = applicationId;
        this.deviceId = deviceId;
        this.merchantId = merchantId;
        this.accessToken = accessToken;
        this.imageUtil = new ImageUtil_1.ImageUtil();
        this.httpSupport = typeof XMLHttpRequest !== 'undefined' ? new HttpSupport_1.HttpSupport(XMLHttpRequest) : null;
        this.webSocketFactoryFunction = BrowserWebSocketImpl_1.BrowserWebSocketImpl.createInstance;
        this.cloverServer = "https://www.clover.com/";
    }
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getApplicationId = function () {
        return this.applicationId;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getDeviceId = function () {
        return this.deviceId;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getMerchantId = function () {
        return this.merchantId;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getAccessToken = function () {
        return this.accessToken;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getCloverServer = function () {
        return this.cloverServer;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.setCloverServer = function (value) {
        this.cloverServer = value;
        return this;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getHttpSupport = function () {
        return this.httpSupport;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.setHttpSupport = function (value) {
        this.httpSupport = value;
        return this;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getWebSocketFactoryFunction = function () {
        return this.webSocketFactoryFunction;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.setWebSocketFactoryFunction = function (value) {
        this.webSocketFactoryFunction = value;
        return this;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getImageUtil = function () {
        return this.imageUtil;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.setImageUtil = function (value) {
        this.imageUtil = value;
        return this;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getFriendlyId = function () {
        return this.friendlyId;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.setFriendlyId = function (value) {
        this.friendlyId = value;
        return this;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getForceConnect = function () {
        return this.forceConnect;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.setForceConnect = function (value) {
        this.forceConnect = value;
        return this;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getHeartbeatInterval = function () {
        return this.heartbeatInterval;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.setHeartbeatInterval = function (value) {
        this.heartbeatInterval = value;
        return this;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getHeartbeatDisconnectTimeout = function () {
        return this.heartbeatDisconnectTimeout;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.setHeartbeatDisconnectTimeout = function (value) {
        this.heartbeatDisconnectTimeout = value;
        return this;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.getReconnectDelay = function () {
        return this.reconnectDelay;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.setReconnectDelay = function (value) {
        this.reconnectDelay = value;
        return this;
    };
    WebSocketCloudCloverDeviceConfigurationBuilder.prototype.build = function () {
        return new WebSocketCloudCloverDeviceConfiguration(this.applicationId, this.webSocketFactoryFunction, this.imageUtil, this.cloverServer, this.accessToken, this.httpSupport, this.merchantId, this.deviceId, this.friendlyId, this.forceConnect, this.heartbeatInterval, this.reconnectDelay, this.heartbeatDisconnectTimeout);
    };
    return WebSocketCloudCloverDeviceConfigurationBuilder;
}());
exports.WebSocketCloudCloverDeviceConfigurationBuilder = WebSocketCloudCloverDeviceConfigurationBuilder;

//# sourceMappingURL=../../../../../maps/com/clover/remote/client/device/WebSocketCloudCloverDeviceConfiguration.js.map
