"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketPairedCloverDeviceConfigurationBuilder = exports.WebSocketPairedCloverDeviceConfiguration = void 0;
var WebSocketPairedCloverTransport_1 = require("../transport/websocket/WebSocketPairedCloverTransport");
var WebSocketCloverDeviceConfiguration_1 = require("./WebSocketCloverDeviceConfiguration");
var ImageUtil_1 = require("../../../util/ImageUtil");
var BrowserWebSocketImpl_1 = require("../../../websocket/BrowserWebSocketImpl");
/**
 * This is the base class that is used when connecting directly to a device via the "Network Pay Display".
 *
 * A pairing scheme is used when connecting, so the function callbacks for when a pairing code is received,
 * and when the pairing completes must be implemented here.
 */
var WebSocketPairedCloverDeviceConfiguration = /** @class */ (function (_super) {
    __extends(WebSocketPairedCloverDeviceConfiguration, _super);
    /**
     *
     * @param {string} endpoint - the endpoint of the Clover device. e.g. wss://192.168.1.15:12345/remote_pay
     * @param {string} applicationId - the applicationId that uniquely identifies the POS. e.g. com.company.MyPOS:2.3.1
     * @param {string} posName - Displayed during pairing to display the POS name on the Mini. e.g. MyPOS
     * @param {string} serialNumber - Displayed during pairing to display the device identifier. e.g. 'Aisle 3' or 'POS-35153234'
     * @param {string} authToken - The authToken retrieved from a previous pairing activity, passed as an argument to onPairingSuccess. This will be null for the first connection.
     * @param onPairingCode - Function that will be called when a pairing code is sent.  Typically, this would be displayed to the user, and
     * they would enter this code into the device screen.
     * @param onPairingSuccess - Function that willbe called when the pairing process is complete, a authentication token is sent that be reused.
     * @param {Object} webSocketFactoryFunction - the function that will return an instance of the CloverWebSocketInterface
     *    that will be used when connecting.
     * @param {IImageUtil} imageUtil - utility to translate images into base64 strings.
     * @param {number} heartbeatInterval - Frequency at which we will ping the device - millis. Defaults to 5000 (millis), minimum value is 2500 millis. To disable the PING set to -1.
     * @param {number} reconnectDelay - upon disconnect, duration to wait until a reconnect is attempted - millis. Defaults to 3000 (millis), minimum value is 3000 millis. To disable the reconnect set to -1.
     * @param {number} heartbeatDisconnectTimeout - If a response to a heartbeat ping is not received within this time we will call disconnect.  Defaults to 3000 (millis).
     */
    function WebSocketPairedCloverDeviceConfiguration(endpoint, applicationId, posName, serialNumber, authToken, onPairingCode, onPairingSuccess, webSocketFactoryFunction, imageUtil, heartbeatInterval, reconnectDelay, heartbeatDisconnectTimeout) {
        var _this = _super.call(this, applicationId, webSocketFactoryFunction, imageUtil, heartbeatInterval, reconnectDelay) || this;
        _this.uri = null;
        _this.uri = endpoint;
        _this.posName = posName;
        _this.serialNumber = serialNumber;
        _this.authToken = authToken;
        _this.webSocketImplClass = webSocketFactoryFunction;
        _this.onPairingCodeHandler = onPairingCode;
        _this.onPairingSuccessHandler = onPairingSuccess;
        return _this;
    }
    WebSocketPairedCloverDeviceConfiguration.prototype.getMessagePackageName = function () {
        return 'com.clover.remote_protocol_broadcast.app';
    };
    WebSocketPairedCloverDeviceConfiguration.prototype.getName = function () {
        return 'Clover Secure WebSocket Connector';
    };
    WebSocketPairedCloverDeviceConfiguration.prototype.getCloverTransport = function () {
        // this is where we determine more about the transport...
        // The connection is paired:
        var transport = new WebSocketPairedCloverTransport_1.WebSocketPairedCloverTransport(this.uri, this.getReconnectDelay(), this.posName, this.serialNumber, this.authToken, this.webSocketImplClass);
        transport.setPairingDeviceConfiguration(this);
        return transport;
    };
    WebSocketPairedCloverDeviceConfiguration.prototype.setAuthToken = function (authToken) {
        this.authToken = authToken;
    };
    WebSocketPairedCloverDeviceConfiguration.prototype.onPairingCode = function (pairingCode) {
        this.onPairingCodeHandler(pairingCode);
    };
    WebSocketPairedCloverDeviceConfiguration.prototype.onPairingSuccess = function (authToken) {
        this.onPairingSuccessHandler(authToken);
    };
    return WebSocketPairedCloverDeviceConfiguration;
}(WebSocketCloverDeviceConfiguration_1.WebSocketCloverDeviceConfiguration));
exports.WebSocketPairedCloverDeviceConfiguration = WebSocketPairedCloverDeviceConfiguration;
var WebSocketPairedCloverDeviceConfigurationBuilder = /** @class */ (function () {
    /**
     * @param {string} applicationId - the applicationId that uniquely identifies the POS. e.g. com.company.MyPOS:2.3.1
     * @param {string} uri - the endpoint of the Clover device. e.g. wss://192.168.1.15:12345/remote_pay
     * @param {string} posName - Displayed during pairing to display the POS name on the Mini. e.g. MyPOS
     * @param {string} serialNumber - Displayed during pairing to display the device identifier. e.g. 'Aisle 3' or 'POS-35153234'
     * @param {string} authToken - The authToken retrieved from a previous pairing activity, passed as an argument to onPairingSuccess. This will be null for the first connection
     * @param onPairingCode - Function that will be called when a pairing code is sent.  Typically, this would be displayed to the user, and
     * they would enter this code into the device screen.
     * @param onPairingSuccess - Function that will be called when the pairing process is complete, a authentication token is sent that be reused.
     */
    function WebSocketPairedCloverDeviceConfigurationBuilder(applicationId, uri, posName, serialNumber, authToken, onPairingCode, onPairingSuccess) {
        this.uri = null;
        this.applicationId = applicationId;
        this.uri = uri;
        this.posName = posName;
        this.serialNumber = serialNumber;
        this.authToken = authToken;
        this.imageUtil = new ImageUtil_1.ImageUtil();
        this.webSocketFactoryFunction = BrowserWebSocketImpl_1.BrowserWebSocketImpl.createInstance;
        this.onPairingCodeHandler = onPairingCode;
        this.onPairingSuccessHandler = onPairingSuccess;
    }
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getApplicationId = function () {
        return this.applicationId;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getUri = function () {
        return this.uri;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getSerialNumber = function () {
        return this.serialNumber;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getAuthToken = function () {
        return this.authToken;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getWebSocketFactoryFunction = function () {
        return this.webSocketFactoryFunction;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.setWebSocketFactoryFunction = function (value) {
        this.webSocketFactoryFunction = value;
        return this;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getImageUtil = function () {
        return this.imageUtil;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.setImageUtil = function (value) {
        this.imageUtil = value;
        return this;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getPosName = function () {
        return this.posName;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.setPosName = function (value) {
        this.posName = value;
        return this;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getHeartbeatInterval = function () {
        return this.heartbeatInterval;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.setHeartbeatInterval = function (value) {
        this.heartbeatInterval = value;
        return this;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getHeartbeatDisconnectTimeout = function () {
        return this.heartbeatDisconnectTimeout;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.setHeartbeatDisconnectTimeout = function (value) {
        this.heartbeatDisconnectTimeout = value;
        return this;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.getReconnectDelay = function () {
        return this.reconnectDelay;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.setReconnectDelay = function (value) {
        this.reconnectDelay = value;
        return this;
    };
    WebSocketPairedCloverDeviceConfigurationBuilder.prototype.build = function () {
        return new WebSocketPairedCloverDeviceConfiguration(this.uri, this.applicationId, this.posName, this.serialNumber, this.authToken, this.onPairingCodeHandler, this.onPairingSuccessHandler, this.webSocketFactoryFunction, this.imageUtil, this.heartbeatInterval, this.reconnectDelay, this.heartbeatDisconnectTimeout);
    };
    return WebSocketPairedCloverDeviceConfigurationBuilder;
}());
exports.WebSocketPairedCloverDeviceConfigurationBuilder = WebSocketPairedCloverDeviceConfigurationBuilder;

//# sourceMappingURL=../../../../../maps/com/clover/remote/client/device/WebSocketPairedCloverDeviceConfiguration.js.map
